#include "helpers.h"
#include <math.h>

// Returns the squared distance between the two points
// This is faster than computing the actual distance because square roots
// are slow. If you need to tell whether one distance is less than another,
// you can always just use the square distance (to make things faster) since
// a < b if and only if sqrt(a) < sqrt(b), so there is no need to call sqrt.
// But just in case you want it, we put a dist function below.
float dist_sq(point_t p, point_t q) {
    float dx = p.x - q.x;
    float dy = p.y - q.y;
    return (dx * dx) + (dy * dy);
}

// Computes the actual distance between p and q. Really not as useful as you might
// think. If you just want to know if one point q is closer to a point p than
// another point r, i.e. if dist(p, q) < dist(p, r), it's better to use the dist_sq
// i.e. just test if dist_sq(p, q) < dist_sq(p, r). Since it doesn't use
// sqrt it is much faster.
float dist(point_t p, point_t q) {
    return sqrt(dist_sq(p, q));
}
