#ifndef KDTREE_H
#define KDTREE_H

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>

#include "helpers.h"
#include "dynarray.h"

////////////////////////////////////////////////////////////////////////////////
// KD-TREE STRUCTURE
////////////////////////////////////////////////////////////////////////////////

typedef struct kdtree_node {
    debris_t debris;
    struct kdtree_node *parent;
    struct kdtree_node *left;
    struct kdtree_node *right;
} kdtree_node_t;

typedef struct kdtree {
    kdtree_node_t *root;    // The root node
    size_t size;            // The number of nodes in the kd-tree
} kdtree_t;


////////////////////////////////////////////////////////////////////////////////
// KD-TREE METHODS (IMPLEMENT THESE!)
////////////////////////////////////////////////////////////////////////////////

typedef kdtree_node_t* kdtree_iterator_t;

// Constructors and memory management:
void kdtree_init(kdtree_t *tree);   // Constructor
void kdtree_free(kdtree_t *tree);   // Frees the kdtree_node_ts stored in the tree
void kdtree_build(kdtree_t *tree, debris_t array[], size_t length);

// Range searching:
void kdtree_box_range_search(kdtree_t *tree, bbox_t range, dynarray_t *debris_in_range);
void kdtree_circ_range_search(kdtree_t *tree, point_t p, float radius, dynarray_t *debris_in_range);


////////////////////////////////////////////////////////////////////////////////
// KD-TREE METHODS (PROVIDED)
////////////////////////////////////////////////////////////////////////////////

kdtree_iterator_t kdtree_iterator_begin(kdtree_t *tree);
kdtree_iterator_t kdtree_iterator_end(kdtree_t *tree);
kdtree_iterator_t kdtree_iterator_next(kdtree_iterator_t it);
debris_t kdtree_iterator_value_at(kdtree_iterator_t it);

#endif
